from __future__ import division
import numpy as np


def _negative_log_likelihood(lambda_rho, T, E):
    if np.any(lambda_rho < 0):
        return np.inf
    lambda_, rho = lambda_rho
    return - np.log(rho * lambda_) * E.sum() - (rho - 1) * (E * np.log(lambda_ * T)).sum() + ((lambda_ * T) ** rho).sum()


def _lambda_gradient(lambda_rho, T, E):
    lambda_, rho = lambda_rho
    return - rho * (E / lambda_ - (lambda_ * T) ** rho / lambda_).sum()


def _rho_gradient(lambda_rho, T, E):
    lambda_, rho = lambda_rho
    return - E.sum() / rho - (np.log(lambda_ * T) * E).sum() + (np.log(lambda_ * T) * (lambda_ * T) ** rho).sum()
    # - D/p - D Log[m t] + (m t)^p Log[m t]


def _d_rho_d_rho(lambda_rho, T, E):
    lambda_, rho = lambda_rho
    return (1. / rho ** 2 * E + (np.log(lambda_ * T) ** 2 * (lambda_ * T) ** rho)).sum()
    # (D/p^2) + (m t)^p Log[m t]^2


def _d_lambda_d_lambda_(lambda_rho, T, E):
    lambda_, rho = lambda_rho
    return (rho / lambda_ ** 2) * (E + (rho - 1) * (lambda_ * T) ** rho).sum()


def _d_rho_d_lambda_(lambda_rho, T, E):
    lambda_, rho = lambda_rho
    return (-1. / lambda_) * (E - (lambda_ * T) ** rho - rho * (lambda_ * T) ** rho * np.log(lambda_ * T)).sum()



'''
Make sure that concordance index is available, even if the Fortran code
has not been compiled.
'''


try:
    from ._cindex import concordance_index
except ImportError:
    print("Using pure Python version of concordance index. \n\
You can speed this up 100x by compiling the Fortran code with:\n\
>>> python setup.py build_ext --inplace")
    from .cindex import concordance_index




import numpy as np
from lifelines import _univariate_weibull as wf


def test_lambda_gradient():

    E = np.array([1])
    T = np.array([10])
    rho = lambda_ = 1
    assert - wf._lambda_gradient([lambda_, rho], T, E) == -9

    E = np.array([1, 1])
    T = np.array([10, 10])
    assert - wf._lambda_gradient([lambda_, rho], T, E) == -9 * 2


def test_rho_gradient():

    E = np.array([1])
    T = np.array([10])
    rho = lambda_ = 1
    assert - wf._rho_gradient([lambda_, rho], T, E) == 1 + 1 * np.log(10) - np.log(10) * 10

    E = np.array([1, 1])
    T = np.array([10, 10])
    assert - wf._rho_gradient([lambda_, rho], T, E) == 2 * (1 + 1 * np.log(10) - np.log(10) * 10)



import os
import sys

# bdist_wheel requires setuptools
import setuptools
# use setup from numpy to build fortran codes
from numpy.distutils.core import setup, Extension


# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


# Fortran extensions for extra speed (optional)
ext_fstat = Extension(name="lifelines._utils._cindex",
                      sources=["lifelines/_utils/_cindex.f90"])
exts = [ext_fstat]


done = False
# First try to build with the extension. In case of failure, do without.
for ext_modules in [exts, []]:
    if done:
        break

    try:
        setup(
            name="lifelines",
            version="0.6.1.0",
            author="Cameron Davidson-Pilon, Jonas Kalderstam",
            author_email="cam.davidson.pilon@gmail.com",
            description="Survival analysis in Python, including Kaplan Meier, Nelson Aalen and regression",
            license="MIT",
            keywords="survival analysis statistics data analysis",
            url="https://github.com/CamDavidsonPilon/lifelines",
            packages=['lifelines',
                      'lifelines.datasets',
                      'lifelines._utils',
                      'lifelines.tests'],
            long_description=read('README.txt'),
            classifiers=[
                "Development Status :: 4 - Beta",
                "License :: OSI Approved :: MIT License",
                "Programming Language :: Python",
                "Topic :: Scientific/Engineering",
                ],
            install_requires=[
                "numpy",
                "scipy",
                "pandas>=0.14",
            ],
            package_data={
                "lifelines": [
                    "*.f90",
                    "../README.md",
                    "../README.txt",
                    "../LICENSE",
                    "../MANIFEST.in",
                    "../*.ipynb",
                    "datasets/*",
                ]
            },
            ext_modules=ext_modules
        )
        done = True
    except:
        print("Failed to build. Trying again without native extensions.")
